/*****************************************************************************************
 *
 *  Copyright 2006+ by XXXXX
 *  All Rights Reserved.
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  0.0        XXXXX         XXXXX              Inital Release
 *
 ****************************************************************************************/

#include <nokix/all.h>

//#define DEBUG

#ifdef DEBUG
#include <stdio.h>
#else
#define printf(...)
#endif

// values for confid_state config id
#define STATE_OFF         0
#define STATE_ON          1

// values for confid_rhythmic config id
#define RHYTHMIC_OFF          0
#define RHYTHMIC_BACKLIGHT    1
#define RHYTHMIC_VIBRA        2

// speed of the blink enabled when system backlight blink starts up
#define BLINK_DELAY_ON        20
#define BLINK_DELAY_OFF       40

extern rexx_uint confid_state;
extern rexx_uint confid_rhythmic;
extern rexx_uint line_id;

extern const struct bitmap bmap_flashlight;

extern const nokstr_t str_flashlight_on;
extern const nokstr_t str_flashlight_off;
//extern const nokstr_t str_unallowed;

REXX_EXPOSE(MSG_CALL_FUNCTION);

//unsigned get_charger_type(void);
void timer_blink_on(void);
static void timer_blink_off(void);

/**************************************************************************************/

// line initialization hooked into start_engine()
void reset_line(void)
{
   own_line_set(line_id,0);
}

// basic flashlight state changing function
unsigned set_flashlight(unsigned on)
{
//   if((on)&&(get_charger_type()!=0)) on=0;
   on = ( on ? STATE_ON : STATE_OFF );
   if( own_get_config(confid_state) == on ) return FALSE;
   own_line_set(line_id,on);
   printf("Flashlight state: %d\n",on);
   own_set_config( confid_state, on );
   return TRUE;
}

// for rhythmic flashlight
static void set_rhythmic_flashlight(unsigned type, unsigned on)
{
   if(own_get_config(confid_rhythmic)==type)
   {
//      if(get_charger_type()==0)
//      {
         if(own_get_config(confid_state)==STATE_OFF)
         {
            own_line_set(line_id,on);
            printf("Flashlight state: %d\n",on);
         }
//      }
   }
}

// for rhythmic flashligth: backlight
void set_external_light(unsigned mode, unsigned on)
{
   set_rhythmic_flashlight(RHYTHMIC_BACKLIGHT,mode==0?on:0);
}

// for rhythmic flashligth: vibra
void hook_set_vibra(unsigned mode)
{
   set_rhythmic_flashlight(RHYTHMIC_VIBRA,((mode==0x40)||(mode==0x60))?1:0);
}

// for blinking
void timer_blink_on(void)
{
   set_external_light(0,1);
   own_timer_start(timer_blink_off,TT_FUNCTION,BLINK_DELAY_ON);
}

// for blinking
static void timer_blink_off(void)
{
   set_external_light(0,0);
   own_timer_start(timer_blink_on,TT_FUNCTION,BLINK_DELAY_OFF);
}

// for blinking
void blink_off(void)
{
   if( (own_timer_abort(timer_blink_on)!=TE_TIMER_NOT_FOUND) ||
       (own_timer_abort(timer_blink_off)!=TE_TIMER_NOT_FOUND) )
   {
      // we set the line to 0 only if there was a timer counting;
      // this is because on phones like 5210 the MSG_LIGHTS_ON and
      // MSG_LIGHTS_OFF (where this function is hooked) are used
      // to make the blink during ringtones playback
      set_external_light(1,0);
   }
}

// for definitions and menu
void flashlight_on(void)
{
   if(set_flashlight(1))
   {
      send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_DONE,&str_flashlight_on);
      send_message(MSG_REDRAW_MENU);
   }
//   else
//   {
//      send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_unallowed);
//   }
}

// for definitions and menu
void flashlight_off(void)
{
   if(set_flashlight(0)) {
      send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_DONE,&str_flashlight_off);
      send_message(MSG_REDRAW_MENU);
   }
}

// for definition
void flashlight_toggle(void)
{
   if(own_get_config(confid_state)==STATE_OFF) flashlight_on();
   else flashlight_off();
}

// hooked into charging start, disables the flashlight
/*void hook_charging_start(unsigned msg, unsigned arg)
{
   if(arg!=0) set_flashlight(0);
}*/

// hooked into UI task, detects "MSG_PHONE_SHUTDOWN"
void hook_task5( unsigned int msg ) {
   if( msg == 0x5E3 ) set_flashlight(0);
}

/* UI messages dispatcher */
rexx("steady_dispatcher_id=add_5e0_id(steady_dispatcher,1)");
extern rexx_uint steady_dispatcher_id;

unsigned steady_dispatcher(unsigned msg)
{
   switch(msg)
   {
      case MSG_D_INIT:
         {
//            unsigned s=own_get_config(confid_state);
            
            if(set_flashlight(1))
            {
//               if(set_flashlight(1))
//               {
                  if(own_timer_query(steady_dispatcher)==TE_TIMER_NOT_FOUND)
                  {
                     own_timer_start(steady_dispatcher,TT_NOP,TICKS_PER_SECOND);

                     freegfx_open();

                     freegfx_begin(true);
                     own_draw_bitmap(&bmap_flashlight,
                        (screen_info.width-bmap_flashlight.width)/2,
                        (screen_info.height-bmap_flashlight.height)/2,
                        DM_SET);
                     freegfx_end(false);

                     msg=MSG_R_STAY;
                  }
                  else send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_DONE,&str_flashlight_on);
//               }
//               else send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_unallowed);
            }
            else //if(s==STATE_ON)
            {
               set_flashlight(0);
               send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_DONE,&str_flashlight_off);
            }
         }
         break;
         
      case MSG_D_QUIT:
      case MSG_KEYRELEASE:
         set_flashlight(0);
         freegfx_close();
         if(msg==MSG_D_QUIT) msg=MSG_R_QUIT;
         else msg=MSG_R_LEAVE;
         break;
   }
   return msg;
}

